;*************************************************************************
;Hexagon 86 v1.1
;TI-86 port by Andreas Finne <a_finne@hotmail.com>
;Original ZShell version by David Lenhart <lenhartd@hotmail.com>
;*************************************************************************


; Hexagon 1.0
; By David Lenhart

; I first saw this game for the IBM PC (spelled HEXXAGON).
; I liked it so much, I thought I'd create it for the TI-85.
; This program uses the first real AI routine I've ever made.
; The best move is calculated by finding the move with the 
; most gain the least immediate consequence.  

; Any added intelligence will probably cause the program to 
; run too slowly, unless you have "turboed" your TI-85 or 
; you can come up with a better algorithm.

#include "ti86asm.inc"

#define pushall push af\ push bc\ push de\ push hl
#define popall	pop hl\ pop de\ pop bc\ pop af


FromXY =        _plotSScreen
FromY =         FromXY
FromX =         FromY+1
self =          FromXY+2
enemy =         self+1
temp =          enemy+1
temp2 =         temp+2
ScoreP1 =       temp2+2
ScoreP2 =       ScoreP1+1
Player1Type =   ScoreP2+1
Player2Type =   Player1Type+1           ;must follow p1type
AddrBoard =     Player2Type+1
BoardNum =      AddrBoard+2
EnemyGain =     BoardNum+1
ThinkMem =      $8200
board =			$8100



.org _asm_exec_ram

	nop
	jp Menu
.dw 1
.dw Title
.dw Icon

Title:
.db "Hexagon 86",0

Icon:
	.db 8,2
	.db %00000000, %00000000
	.db %00011111, %11110000
	.db %00100000, %00001000
	.db %01000010, %10000100
	.db %10000001, %00000010
	.db %01000010, %10000100
	.db %00100000, %00001000
	.db %00011111, %11110000

Menu:
		call _flushallmenus
		call _runindicoff
        ld c, 4
MenuStart:
        push bc
        call _clrLCD
        ld hl, $0401
        ld (_curRow), hl
        ld hl, HexTitle
        call _puts
        ld bc, ($100*(24-6))+45
        ld l, (6*13)+12
        ld h, l
        call Horiz
        inc c
        inc c
        ld h, l
        call Horiz
        ld a, c
        add a, 10
        ld c, a
        ld h, l
        call Horiz
        inc c
        inc c
        ld h, l
        call Horiz
        
        ld hl, ($100*19)+36
        ld (_penCol), hl
        ld hl, byDavidLenhart
        call _vputs
		ld bc,25*256+19
		ld (_penCol),bc
		call _vputs
        pop bc
MenuL:        
        push bc
DrawMenu:
        ld hl, MenuStr
        ld b, 4
MenuLoop:        
        push bc
        ld a, $03
        ld (_curCol), a
        ld a, 8
        sub b
        ld (_curRow), a
        ld a, b
        cp c
        jr nz, itt
        set 3, (IY+5)
itt:
        push hl
        call _puts
        pop hl
        ld de, 16
        add hl, de
        cp c
        jr nz, ittt
ittt:   
        res 3, (IY+5)
        pop bc
        djnz MenuLoop


        pop bc

        call GetKeyNZ
        push af
        cp 1
        jr nz, __NotDown
        ld a, c
        cp 1
        jr z, __NotDown
        dec c
__NotDown:
        pop af
        push af        
        cp 4
        jr nz, __NotUp
        ld a, c
        cp 4
        jr z, __NotUp
        inc c
        ld a, c
Spiffy:
__NotUp:
        pop af
        cp $36  ;2nd
        jr nz, _Not2nd
        ld a, c
        push bc
        cp 1
        jr z, NoStartGame
        ld hl, Player1Type
        ld (hl), 0              ; 1=human  0=ti86
        inc hl
        ld (hl), 1
        cp 2
        jr nz, Not_C_C
        ld (hl), 0
Not_C_C:
        cp 3
        jr nz, Not_H_H
        dec hl
        ld (hl), 1
Not_H_H:
        call GameStart
        xor a                   ; so won't exit or editboard
NoStartGame:
        cp 1
        call z,EditBoard
        pop bc
        xor a                      ;so it won't exit
        jp MenuStart
_Not2nd:
        cp $37
        jp nz,MenuL

SaveBoardChanges:
	ld hl,_asapvar
	rst 20h
	rst 10h
	ld a,b
	ld hl,BoardSave-_asm_exec_ram+4
	add hl,de
	adc a,0
	call _set_abs_dest_addr
	xor a
	ld hl,BoardSave
	call _set_abs_src_addr
	ld hl,81
	call _set_mm_num_bytes
	jp _mm_ldir		

        
GameStart:

        ld hl, board
        ld (AddrBoard), hl
        call ResetBoard
        call DrawBoard
        ld a, 2
        ld (enemy), a
        ld a, 1
        ld (self), a
GameLoop:
        ld bc, ($100*117)+1
        push bc
        ld h, 8
Loop:
        call ORdot
        inc b
        dec h
        jr nz, Loop
        pop bc
        ld a, (enemy)
        dec a
        jr z, Zeroo
        ld a, 5
Zeroo:
        add a, b
        ld b, a
        ld h, 3
        call Horiz

        ld b, 120
        ld h, 2
        call Horiz

        ld hl, Player1Type
        ld a, (self)  ;switchplayers already did this
        cp 2
        jr nz, PlayerOne
        inc hl
PlayerOne:
        ld a, (hl)      ;playertype
        or a                            ;computer=0, human=1
        push af
        call z,CPUmove
        pop af
        call nz,HumanMove
		ld a,($C006)
        cp $37
        jr z, exitgame
        ld a, (IX)
        cp -10
        jp z,GameOver

        call DoAttack
        call DrawScores
        call SwitchPlayers
        jr GameLoop
exitgame:
        call _getky
        ret

HumanMove:
        ld IX, ThinkMem
        ld a, 1
        ld (BoardNum), a
        call NewLevel
        ld a, (IX)
        cp -10
        call nz,DoMove
        ret

FindTile:
; x=11x+8
; y=6y+3x-13
        ld h, b
        ld l, c

        ld b, h
        ld c, 11
        call multiply
        add a, 8
        ld d, a

        ld b, l
        ld c, 6
        call multiply
        ld e, a
        ld b, 3
        ld c, h
        call multiply
        add a, e
        sub 13
        ld c, a
        ld b, d
        ret



DrawHex:
        pushall
        call FindTile
        ld a, b
        sub 7
        ld b, a
        call ORdot
        push bc
        inc b
        dec c
        call ORdot
        inc b
        dec c
        call ORdot
        pop bc

        inc b
        inc c
        call ORdot
        inc b
        inc c
        call ORdot
        inc c
        inc b
        ld h, 9
LineLoop:
        call ORdot
        push bc
        ld a, c
        sub 6
        ld c, a
        call ORdot
        pop bc
        inc b
        dec h
        jr nz, LineLoop
        dec c
        call ORdot
        inc b
        dec c
        call ORdot
        inc b
        dec c
        call ORdot
        dec b
        dec c
        call ORdot
        dec b
        dec c
        call ORdot
        popall
        ret



divide:                 ; c=a/b   a+b=remainder
        ld c, 0
DivLoop:
        sub b
        ret m
        inc c
        jr DivLoop



DoMove:
        xor a
        ld (FromX), a           ;no piece to move yet
        ld b, 5
        ld c, 5
        call DrawCursor
WaitForKey:
		push bc        
        call _getky
		pop bc
        or a
        jr z, WaitForKey
        cp $37
        jr nz, NotExit
        pop af;         trash return addr
        pop af;         trash return addr
        ret             ; exit to menu

NotExit:
        cp $36                  ;2nd
        push af
        jr nz, Not2nd
        call FindPiece
        ld a, (self)
        cp (hl)
        jr nz, NotSelf
        ld (FromXY), bc
        jr SkipMove
NotSelf:
        xor a                   ;blank space=0
        cp (hl)
        jr nz, NotBlank
        ld a, (FromX)
        or a
        jr z, NoPieceToMove
        ld de, (FromXY)
        ld a, b
        sub d
        jr nc, Skp
        neg
Skp:
        sub 3
        jr nc, TooFar
        ld a, c
        sub e
        jr nc, Skp2
        neg
Skp2:
        sub 3
        jr nc, TooFar
        ld a, b
        sub d
        add a, c
        sub e
        bit 7, a
        jr z, positive
        neg
positive:
        sub 3
        jr nc, TooFar
        ;bc = ToXY
        ;;;ld de, (FromXY)
        call DrawCursor
        pop af          ; lower sp before returning
        ret
TooFar:
NoPieceToMove:
NotBlank:
SkipMove:
Not2nd:
        call DrawCursor
        pop af
        
        
        
        
        call KeyboardMove
        
        call DrawCursor
        jp WaitForKey

DrawObstacle:
        pushall
        call FindTile
        dec b
        dec b
        dec b
        dec b
        inc c
        inc c
        ld h, 9
        call DottedHoriz
        dec b
        dec c
        ld h, 11
        call DottedHoriz
        dec b
        dec c
        ld h, 13
        call DottedHoriz
        inc b
        dec c
        ld h, 11
        call DottedHoriz
        inc b
        dec c
        ld h, 9
        call DottedHoriz

        popall
        ret




DrawP1:
        pushall
        call FindTile
        call Dp1
        popall
        ret

Dp1:
        inc c
        call XORdot
        dec b
        call XORdot
        dec c
        call XORdot
        dec c
        call XORdot
        inc b
        call XORdot
        inc b
        call XORdot
        inc c
        call XORdot
        inc c
        call XORdot
        ret

DrawP2:
        pushall
        call FindTile
        call dp2
        popall
        ret
        
dp2:        
        call XORdot
        dec b
        inc c
        call XORdot
        inc b
        inc b
        call XORdot
        dec c
        dec c
        call XORdot
        dec b
        dec b
        call XORdot
        ret


DrawCursor:
        pushall
        call FindTile
        dec b
        dec b
        dec b
        dec b
        inc c
        inc c
        ld h, 9
        call Horiz
        dec b
        dec c
        ld h, 11
        call Horiz
        dec b
        dec c
        ld h, 13
        call Horiz
        inc b
        dec c
        ld h, 11
        call Horiz
        inc b
        dec c
        ld h, 9
        call Horiz
        popall
        ret


DoAttack:
                        ; bc = to XY    de = from XY
        push bc
        ld b, d
        ld c, e
        call DrawCursor
        pop bc
        ld h, 6
FlashMove:        
        call DrawCursor
        push bc
        ld bc, $6000
        call delay
        pop bc
        dec h
        jr nz, FlashMove
        push bc
        ld b, d
        ld c, e
        call DrawCursor
        pop bc



        call CheckRange
        jr z, _Reproduce_
        push bc
        ld b, d
        ld c, e
        call FindPiece
        xor a
        ld (hl), a
        ld a, (self)
        cp 1
        call z,DrawP1
        call nz,DrawP2
        pop bc
        jr ReproduceSkip
_Reproduce_:
        ld a, (self)
        dec a                   ; cp 1
        jr nz, p2reproduce
        ld hl, ScoreP1
        inc (hl)
        jr p2reproduceSkip
p2reproduce:
        ld hl, ScoreP2
        inc (hl)
p2reproduceSkip:
ReproduceSkip:
        ld a, (self)             ; move to new space
        dec a                   ;cp 1
        call z,DrawP1
        call nz,DrawP2
        call FindPiece
        ld a, (self)
        ld (hl), a

        
        
        ld d, b
        ld e, c         ; for checkrange

        inc b
        inc c
        push bc
        ld a, 6
KillFlash:        
        push af
        push bc
        ld bc, $6000
        call delay
        pop bc
        push bc
        ld l, 3
yL:
        push bc
        ld h, 3
xL:
        call CheckRange
        jr nz, NoDrw
        call CheckOnBoard
        jr nz, NoDrw
        push hl
        call FindPiece
        ld a, (enemy)
        cp (hl)
        call z,DrawCursor
        pop hl
NoDrw:
        dec b
        dec h
        jr nz, xL
        pop bc
        dec c
        dec l
        jr nz, yL
        pop bc
        pop af
        dec a
        jr nz, KillFlash
        pop bc



        ld l, 3
yL_:
        ld h, 3
        push bc
xL_:
        call CheckOnBoard
        jr nz, NoDrw_
        call CheckRange
        jr nz, NoDrw_
        push hl
        call FindPiece
        ld a, (enemy)
        cp (hl)
        jr nz, No_Enemy
        ld a, (self)
        ld (hl), a
        dec a           ; cp 1
        push af
        jr nz, p2inc
        ld hl, ScoreP1
        inc (hl)
        ld hl, ScoreP2
        dec (hl)
        jr p2incSKip
p2inc:
        ld hl, ScoreP1
        dec (hl)
        ld hl, ScoreP2
        inc (hl)
p2incSKip:
        pop af
        call z,DrawP2
        call DrawP1
        call nz,DrawP2
No_Enemy:
        pop hl
NoDrw_:
        dec b
        dec h
        jr nz, xL_
        pop bc
        dec c
        dec l
        jr nz, yL_
        ret


CPUmove:
        call _runindicon
        ld IX, ThinkMem
        xor a
        ld (BoardNum), a
        call NewLevel         ; Recursive
        call _runindicoff
        ld b, (IX+10)
        ld c, (IX+20)
        ld d, (IX+30)
        ld e, (IX+40)
        ret

NewLevel:
        ld (IX), -10              ; gain
        ld c, 9
LoopY:
        ld b, 9
LoopX:
        ld (IX+50), b
        ld (IX+60), c
        call FindPiece
        ld a, (self)
        cp (hl)
        jr nz, NoPiece
        ld e, -2
        ld l, 5
LoopY2:
        ld d, -2
        ld h, 5
LoopX2:
        push hl
        ld a, d
        add a, e
          bit 7, a
        jr z, NoNeg
        ;jr nc, NoNeg
        neg
NoNeg:
        sub 3
        jr nc, NotInRange
        push bc
        ld a, c
        add a, e
        ld c, a
        ld a, b
        add a, d
        ld b, a

        call CheckOnBoard     ;z=true
        jr nz, OffBoard
        call FindPiece        ;hl, a
        xor a                   ;ld a, 0  = blank space
        cp (hl)
        push de
        call z,Stuff_
        pop de
OffBoard:
        pop bc
NotInRange:
        pop hl
        inc d
        dec h
        jr nz, LoopX2
        inc e
        dec l
        jr nz, LoopY2
NoPiece:
        djnz LoopX
        dec c
        jr nz, LoopY
        ret


Stuff_:
        ld d, (IX+50)
        ld e, (IX+60)
        call FindGain
        ld a, e
        sub (IX)
        dec a
        ;;bit 7, a
        ;;ret nz
        ret m                   ; if not better without deducting enemy gain,
                                ; then the score cannot possibly be better.


        ld a, (BoardNum)
        cp 1
        jr z, NoFurther
                push bc
                push de         ;save e
NewBoard:
        push bc
        ld hl, BoardNum
        inc (hl)
        ld hl, (AddrBoard)
        ld bc, 81
        push hl
        add hl, bc
        ex de, hl
        pop hl
        ldir
        ld (AddrBoard), hl
        pop bc
                ld d, (IX+50)
                ld e, (IX+60)
                call SilentAttack
                call SwitchPlayers
                inc IX
                call NewLevel
                dec IX
                call SwitchPlayers
OldBoard:
        ld hl, (AddrBoard)
        ld bc, -81
        add hl, bc
        ld (AddrBoard), hl
        ld hl, BoardNum
        dec (hl)
                pop de
                pop bc
                ld a, e
                sub (IX+1)
                ld e, a
NoFurther:
        ld a, e
        sub (IX)
        ret m

                        ;############# d = friends ####################
CountFriends:           ;###############################################
        push af
        push bc
        push de

        ld b, (IX+50)   ;Current From X
        ld c, (IX+60)   ;Current From Y

        ld d, -1        ; you can't count yourself as a friend
        inc b
        inc c
        ld l, 3
__L_y:
        ld h, 3
        push bc
__L_x:
        push hl
        call CheckOnBoard
        jr nz, OffB
        push de
        ld d, (IX+50)   ;Current From X
        ld e, (IX+60)   ;Current From Y
        call CheckRange
        pop de
        jr nz, NotVal          ; z = close
        call FindPiece
        ld a, (self)
        cp (hl)
        jr nz, NotFriend
        inc d
NotFriend:
OffB:
NotVal
        dec b
        pop hl
        dec h
        jr nz, __L_x
        pop bc
        dec c
        dec l
        jr nz, __L_y
        ld a, d
        pop de
        ld d, a
        pop bc
        pop af

        ;;flag not change
        
        jr nz, NotSameButBetter
TheSame:
        
        ld a, d         ; friends
        sub (IX+70)
        
        
        jr c, LessFriends
MoreOrEqual:
        ret nz            ; return not equal  (more)
Equal:
        call CheckEdge        ;if on edge, then usually a better move
        or a
        ret z
        ld a, r
        bit 1, a
        ret z             ; random ret
LessFriends:

NotSameButBetter:
        ld (IX), e      ;net gain
        ld (IX+10), b   ;toX
        ld (IX+20), c   ;toY
        ld (IX+70), d   ;friends nearby
        ld b, (IX+50)   ;Current From X
        ld c, (IX+60)   ;Current From Y
        ld (IX+30), b
        ld (IX+40), c
        ret



DrawBoard:        
        pushall
        xor a
        ld (ScoreP1), a
        ld (ScoreP2), a
        pushall
        call _clrLCD
        popall
        ld c, 9
yLoop:
        ld b, 9
xLoop:
        push bc
        ld b, 9
        call multiply
        pop bc
        add a, b
        sub 9+1
        ld hl, board
        ld d, 0
        ld e, a
        add hl, de
        ld a, (hl)
        cp 7
        call nz,DrawHex
        cp 1
        jr nz, np1
        call DrawP1
        ld hl, ScoreP1
        inc (hl)
np1:
        cp 2
        jr nz, np2
        call DrawP2
        ld hl, ScoreP2
        inc (hl)
np2:
        cp 3
        call z,DrawObstacle
        djnz xLoop
        dec c
        jr nz, yLoop
        ld b, 118
        ld c, 4
        call Dp1
        ld b, 123
        dec c
        call dp2
        call DrawScores
        popall
        ret

SwitchPlayers:
        ld hl, enemy
        ld a, (hl)
        ld b, a
        ld a, (self)
        ld (hl), a
        ld a, b
        ld (self), a
        ret


SilentAttack:                 ;slower
                ; bc= to
                ; de= from
        call FindPiece
        ld a, (self)
        ld (hl), a
        call CheckRange       ;z=close
        jr z, Repr
        push bc
        ld b, d
        ld c, e
        call FindPiece
        pop bc
        xor a
        ld (hl), a              ; erase piece if jump
Repr:
        ld d, b
        ld e, c
        inc b
        inc c
        ld l, 3
_L_y:
        ld h, 3
        push bc
_L_x:
        push hl
                ;de = toXY
        call CheckRange       ;z=close
        jr nz, _NotValid
        call CheckOnBoard     ;z=true
        jr nz, _Not_On
        call FindPiece
        ld a, (enemy)
        cp (hl)
        jr nz, _No_Enemy_There
        ld a, (self)
        ld (hl), a
_No_Enemy_There:
_Not_On:
_NotValid:
        pop hl
        dec b
        dec h
        jr nz, _L_x
        pop bc
        dec c
        dec l
        jr nz, _L_y
        ret



FindGain:                            ;Slower routine
                ; bc= to
                ; de= from
        push bc
        ld (temp), bc

        call CheckRange       ;z=close
        ld e, 0
        jr nz, SkipRepr
        inc e
SkipRepr:
        inc b
        inc c
        ld l, 3
L_y:
        ld h, 3
        push bc
L_x:
        push hl
        push de
        ld de, (temp)
        call CheckRange
        pop de
        jr nz, Not_On
        call CheckOnBoard     ;z=true
        jr nz, Not_On
        call FindPiece
        ld a, (enemy)
        cp (hl)
        jr nz, No_Enemy_There
        inc e
No_Enemy_There:
Not_On:
        pop hl
        dec b
        dec h
        jr nz, L_x
        pop bc
        dec c
        dec l
        jr nz, L_y
        pop bc
        ret             ;e=gain


ResetBoard:
        ld hl, BoardSave
        ld de, board
        ld bc, 9*9
        ldir
        xor a
        ld (BoardNum), a
        ld hl, board
        ld (AddrBoard), hl
        ret


EditBoard:
        call ResetBoard
        call DrawBoard
        ld bc, $0505
        call DrawCursor
EditLoop:
        call GetKeyNZ
        
        cp $35
        jr nz, NotF1
        call EraseOldPiece
        ld (hl), 1
        call DrawP1
        push hl
        ld hl, ScoreP1
        inc (hl)
        push bc
        call DrawScores
        pop bc
        pop hl
NotF1:
        cp $34
        jr nz, NotF2
        call EraseOldPiece
        ld (hl), 2
        call DrawP2
        push hl
        ld hl, ScoreP2
        inc (hl)
        push bc
        call DrawScores
        pop bc
        pop hl
NotF2:
        cp $33
        jr nz, NotF3
        call EraseOldPiece
        ld (hl), 3
        call DrawObstacle
NotF3:
        cp $32
        jr nz, NotF4
        call EraseOldPiece
        ld (hl), 0
NotF4:
        call DrawCursor
        cp $37
        jr nz, NoReturn_
SaveTheBoard:
        ld de, BoardSave
        ld hl, board
        ld bc, 9*9
        ldir             ;copy board to location within program
        ret                       ;exit editor
NoReturn_        
        call KeyboardMove
        call DrawCursor
        jp EditLoop
        ret

EraseOldPiece:
        push af
        call FindPiece
        ld a, 1
        cp (hl)
        jr nz, Not_P1_
        call DrawP1
        push hl
        ld hl, ScoreP1
        dec (hl)
        pop hl
Not_P1_:
        inc a
        cp (hl)
        jr nz, Not_P2_
        call DrawP2
        push hl
        ld hl, ScoreP2
        dec (hl)
        pop hl
Not_P2_:
        inc a
        cp (hl)
        call z,DrawObstacle
        pop af
        push bc
        call DrawScores
        pop bc
        ret



KeyboardMove:           ; a=keyboard scancode, bc=tile posXY
        cp 1
        push af
        jr nz, NotDown
        dec c
        call CheckOnBoard       ; nz = off of board
        jr z, OkDown
        inc c
OkDown:
NotDown:
        pop af
        cp 4
        push af
        jr nz, NotUp
        inc c
        call CheckOnBoard       ; nz = off of board
        jr z, OkUp
        dec c
OkUp:
NotUp:
        pop af
        cp 2
        push af
        jr nz, NotLeft
        dec b
        call CheckOnBoard       ; nz = off of board
        jr z, OkLeft
        inc b
OkLeft:
NotLeft:
        pop af
        cp 3
        ret nz
        inc b
        dec c
        call CheckOnBoard       ; nz = off of board
        ret z
        dec b
        inc c
        ret
        


GameOver:
        call SwitchPlayers
        or 1
        ld (temp2), a
FillBoard:
        ld a, 1
        ld (BoardNum), a
        ld IX, ThinkMem
        call NewLevel
        ld a, (IX)
        cp -10
        jr z, DoneFill
        ld b, (IX+10)
        ld c, (IX+20)
        ld d, (IX+30)
        ld e, (IX+40)
        call CheckRange
        jr nz, DoneFill                 ; no jumps permitted
		ld a,($C006)
        cp K_EXIT                          ; exit
        jr nz, DoFancy
        xor a
        ld (temp2), a
DoFancy:
        ld a, (temp2)
        or a
        push af
        call nz,DoAttack
        pop af
        call z,SilentAttack
        call DrawScores
        jr FillBoard
DoneFill:
        ld a, (temp2)
        or a
        call z,DrawBoard
        
        ld hl, $FC00
InvertColor:
        ld a, (hl)
        cpl
        ld (hl), a
        inc hl
        bit 2, h
        jr nz, InvertColor

        call GetKeyNZ
        ret             ; ret to menu


ORdot:
        push hl
		push bc
		ld a, $3f	;  I
		sub c		;  I
		ld c, a		;  I
        call Find_Pixel
        or (hl)
        ld (hl), a
		pop bc
        pop hl
        ret

XORdot:
        push hl
		push bc
		ld a, $3f	;  I
		sub c		;  I
		ld c, a		;  I
        call Find_Pixel
        xor (hl)
        ld (hl), a
		pop bc
        pop hl
        ret

HexTitle:
.db "H E X A G O N",0
MenuStr:
.db " Human v TI-86 ",0
.db " Human v Human ",0
.db " TI-86 v TI-86 ",0
.db " Board Editor  ",0
byDavidLenhart:
.db "by David Lenhart",0
PortTxt:
.db "TI-86 port by Andreas Finne",0

multiply:               ; a=b*c
        ld a, b
        or a
        ret z
        xor a           ; a=0
MultLoop:
        add a, c
        djnz MultLoop
        ret

FindPiece:                      ; 21 bytes
        ld hl, (AddrBoard)
        push bc
        ld b, c         ; faster
        ld c, 9
        call multiply
        pop bc
        add a, b
        sub 9+1
        push de
        ld d, 0
        ld e, a
        add hl, de
        pop de
        ret

; 39 bytes
CheckRange:             ; returns z if move is close, otherwise nz
        ld a, e
        sub c
        add a, d
        sub b
        bit 7, a
        jr z, NoNeg_3
        neg
NoNeg_3:
        sub 2           ; a must be 2
        jr z, Nope
        ld a, d
        sub b
        jr nc, NoNeg_1
        neg
NoNeg_1:
        cp 2                    ;>1
        jr nc, Nope
        ld a, e
        sub c
        jr nc, NoNeg_2
        neg
NoNeg_2:
        sub 2                   ;>1
        jr nc, Nope
        xor a
        ret
Nope:
        or 1
        ret

; 30 bytes
CheckOnBoard:           ; returns z if on board, nz otherwise
        ld a, b
        ;;cp 1
        ;;jr c, OffBoard_
          or a
          jr z, OffBoard_
        cp 9+1
        jr nc, OffBoard_
        add a, c
        cp 6
        jr c, OffBoard_
        cp 14+1
        jr nc, OffBoard_
        ld a, c
        ;;cp 1
        ;;jr c, OffBoard_
          or a
          jr z, OffBoard_
        cp 9+1
        jr nc, OffBoard_
        xor a
        ret
OffBoard_:
        or 1
        ret

Horiz:         ; 10 bytes
        push bc
HorizLoop_:
        call XORdot
        inc b
        dec h
        jr nz, HorizLoop_
        pop bc
        ret

delay:         ;6 bytes
DelayLoop:        
        dec bc
        ld a, b
        or c
        jr nz, DelayLoop
        ret

CheckEdge:              ; b or c can't be 0 so:  (a=nz)=on edge  (a=z)=not on edge
        ld a, b         ;23 bytes
        cp 1
        ret z
        cp 9
        ret z
        add a, c
        cp 6
        ret z
        cp 14
        ret z
        ld a, c
        cp 1
        ret z
        cp 9
        ret z
        xor a   ;not on edge
        ret

DrawScores:             ;39 bytes
        ld b, 123
        ld c, 63
ScoreLoop:
        call ORdot
        ld b, 118
        call ORdot
        ld a, (ScoreP1)
        add a, 6
        sub c
        call c, XORdot
        ld a, (ScoreP2)
        add a, 6
        sub c
        ld b, 123
        call c, XORdot
        dec c
        ld a, c
        sub 6
        jr nz, ScoreLoop
        ret

GetKeyNZ:
GetKeyNZL:              ;7 bytes
		push bc
		push de
        call _getky
		pop de
		pop bc
        or a
        ret nz
        jr GetKeyNZL

DottedHoriz:         ; 14 bytes
        push bc
HorizLoop:
        call XORdot
        inc b
        inc b
        dec h
        dec h
        bit 7, h
        jr z, HorizLoop
        pop bc
        ret

BoardSave:
.db 7,7,7,7,1,0,0,0,2
.db 7,7,7,0,0,0,0,0,0
.db 7,7,0,0,0,0,0,0,0
.db 7,0,0,0,3,0,3,0,0
.db 2,0,0,0,0,0,0,0,1
.db 0,0,3,0,3,0,0,0,7
.db 0,0,0,0,0,0,0,7,7
.db 0,0,0,0,0,0,7,7,7
.db 1,0,0,0,2,7,7,7,7


;------------------------------------------------------
;   FIND PIXEL ROUTINE by CLEM <clems@mygale.org>
;
;       Input:  x->b
;               y->c
;
;       Output: hl : byte in LCD memory
;               a  : bitmask for (hl)
;
;   121 cycles  27 bytes    Destroyes : none
;------------------------------------------------------

Find_Pixel:
    ld h,63
    ld a,c
    add a,a
    add a,a
    ld l,a
    ld a,b
    rra
    add hl,hl
    rra
    add hl,hl
    rra
    or l
    ld l,a
    ld a,b
    and 7
    cpl
    rlca
    rlca
    rlca
    ld (FP_Bit+1),a
    xor a
FP_Bit:
    set 0,a
    ret



.end
